var dB					= new air.SQLConnection();

// ------------ doSQLLoad ------------
function doSQLLoad(){
	var dbFile 			= air.File.applicationStorageDirectory.resolvePath("shifd.db");
	// opener listener
	dB.addEventListener( air.SQLEvent.OPEN, doDbOpen );
	// open
	dB.open(dbFile);
}

// ------------ doSQLUnload ------------
function doSQLUnload()
{
	dB.close();
}

// ------------ doDbOpen ------------
function doDbOpen( event ){
	// in sqlSchema.js
	doCreateSchema();
}

// ------------ doStmtError ------------
function doStmtError( event )
{
	doAlert( 'global SQL error' );
}

// =========================================
// =============== VERSION =================
// =========================================
// ------------ sqlVersionGet ------------
function sqlVersionGet(){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'Version query successful';

	stLocal.text			= "SELECT version, dateStamp ";
	stLocal.text			= stLocal.text + "FROM version ";

	try {
		stLocal.stmt = sqlRun(stLocal.text);
	}
	catch(e){
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}

	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0];
	} else {
		stReturn.success		= false;
	}
	return stReturn;
}

// ------------ sqlVersionSet ------------
function sqlVersionSet( version ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'Version updated succesfully';


	stLocal.stCurrentVersion	= sqlVersionGet();

	// update
	if( stLocal.stCurrentVersion.success ){
		stLocal.text			= "UPDATE version SET version = :version";
	}
	// insert
	else{
		stLocal.text			= "INSERT INTO version ( version ) VALUES (:version)";
	}

	try {
		stLocal.stmt = sqlRun(stLocal.text, {version: version});
	}catch(e) {
		debug("Error: sqlVersionSet: " + e);
		stReturn.success			= false;
		stReturn.message			= 'Database error';
	}
}

// =========================================
// =============== WINDOWS =================
// =========================================
// ------------ sqlWindowGet ------------
function sqlWindowGet( name ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'Window query successful';


	stLocal.text				= "SELECT name, width, height, x, y ";
	stLocal.text				= stLocal.text + "FROM window ";
	stLocal.text				= stLocal.text + "WHERE name = :name";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {name: name});
	}
	catch(e){
		debug("Error: sqlWindowGet: " + e);
		stReturn.success		= false;
		stReturn.message		= "Database error";
	}


	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0];
	}else{
		stReturn.success		= false;
	}
	return stReturn;
}

// ------------ sqlSaveWindow ------------
function sqlSaveWindow( name, width, height, x, y ){
	var stLocal					= {};

	stLocal.windowExists		= sqlWindowGet( name );

	// update
	if( stLocal.windowExists.success == true ){
		stLocal.text			= "UPDATE window SET width = :width, "
											+ "height = :height, "
											+ "x = :x, "
											+ "y = :y "
			+ "WHERE name = :name";
	}
	// insert
	else{
		stLocal.text			= "INSERT INTO window ";
		stLocal.text			= stLocal.text + "( name, width, height, x, y ) ";
		stLocal.text			= stLocal.text + "VALUES ";
		stLocal.text			= stLocal.text + "(:name, "
											+ ":width, "
											+ ":height, "
											+ ":x, "
											+ ":y )";
	}

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
			name: name,
			width: width,
			height: height,
			x: x,
			y: y
		});
	}catch(e) {
		debug("Error: sqlSaveWindow: " + e);
	}
}

// =========================================
// ================= USER ==================
// =========================================
// ------------ sqlUserGet ------------
function sqlUserGet( username ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'User query successful';


	stLocal.text			= "SELECT userId, userName, fname, lname, email, password, firstTimeArchiving, CASE WHEN lastSync IS NOT NULL THEN CAST(datetime( lastSync ) as varchar) ELSE '' END as lastSync "
							 		+ "FROM user "
							 		+ "WHERE username = :username ";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {username:username});
	}
	catch(e){
		debug("Error: sqlUserGet: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}

	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0];
	} else {
		stReturn.success		= false;
		stReturn.message		= 'Invalid username: ' + username;
	}
	return stReturn;
}

// ------------ sqlUserSave ------------
function sqlUserSave( stUser ){
	var stLocal					= {};

	stUser.password				= doHash(stUser.password);

	stLocal.text				= "SELECT * FROM user WHERE username = :username" ;

	stLocal.stmt 				= sqlRun(stLocal.text, {username:stUser.username});

	stLocal.parameters 			= {}
	try {
		stLocal.rc				= stLocal.stmt.getResult().data.length;
		stLocal.text			= "UPDATE user SET username = username "
									+ ",fname = :fname "
									+ ",lname = :lname "
									+ ",email = :email ";

		stLocal.parameters['fname'] = stUser.fname;
		stLocal.parameters['lname'] = stUser.lname;
		stLocal.parameters['email'] = stUser.email;

		if( typeof stUser.password != 'undefined' ){
			stLocal.text	+= ",password = :password "
			stLocal.parameters['password'] = stUser.password;
		}
		if( typeof stUser.lastSync != 'undefined' ){
			stLocal.text	+= ",lastSync = :lastSync "
			stLocal.parameters['lastSync'] = stUser.lastSync;
		}
		if( typeof stUser.firstTimeArchiving != 'undefined' ){
			stLocal.text	+= ",firstTimeArchiving = :firstTimeArchiving "
			stLocal.parameters['firstTimeArchiving'] = stUser.firstTimeArchiving;
		}
		stLocal.text		+=  "WHERE username = :username";

		stLocal.parameters['username'] = stUser.username;

	} catch(e) {
		stLocal.text		= "INSERT INTO user "
								+ "( username, fname, lname, email, password ) "
								+ "VALUES "
							    + "(:username, "
								+ ":fname, "
								+ ":lname, "
								+ ":email, "
								+ ":password )";
		stLocal.parameters = {};
		stLocal.parameters['username'] = stUser.username;
		stLocal.parameters['fname'] = stUser.fname;
		stLocal.parameters['lname'] = stUser.lname;
		stLocal.parameters['email'] = stUser.email;
		stLocal.parameters['password'] = stUser.password;

	}
	try {
		stLocal.stmt = sqlRun(stLocal.text, stLocal.parameters);
	}catch(e) {
		debug("Error: sqlUserSave: " + e);
	}
}

// ------------ sqlUserValidate ------------
function sqlUserValidate( username, password ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'Is Valid User';

	password					= doHash(password);

	stLocal.text			= "SELECT userId, userName, fname, lname, email "
							 	+ "FROM user "
							 	+ "WHERE username = :username "
							 	+ "AND password = :password ";


	try {
		stLocal.stmt = sqlRun(stLocal.text, {username:username, password:password});
	} catch(e) {
		debug("Error: sqlUserValidate: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0];
	} else {
		stReturn.success		= false;
		stReturn.message		= 'Invalid User';
	}
	return stReturn;
}

// ------------ sqlUserGetLastSync ------------
function sqlUserGetLastSync( userid, bufferMinutes ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'User query successful';


	stLocal.myOffset			= new Date().getTimezoneOffset() / (-60);

	if( typeof bufferMinutes == 'undefined' ){
		bufferMinutes			= 0;
	}

	stLocal.text			= "SELECT CASE WHEN lastSync IS NOT NULL THEN CAST(datetime( lastSync ,'" + bufferMinutes + " minutes'  ) as varchar) ELSE '' END as lastSync, CASE WHEN lastSync IS NOT NULL THEN CAST(datetime( lastSync, '" + stLocal.myOffset + " hours' ) as varchar) ELSE '' END as lastSyncLocal "
							 + "FROM user "
							 + "WHERE userid = :userid ";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
				userid: userid
				});
	}
	catch(e){
		debug("Error: sqlUserGetLastSync: " + e);
		stReturn.success		= false;
		stReturn.message		= 'User query failed';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0];
	} else {
		stReturn.success		= false;
	}
	return stReturn;
}

// ------------ sqlUserUpdateLastSync ------------
function sqlUserUpdateLastSync( userid ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'User Last Sync Updated';

//	stLocal.today				= new Date();

	stLocal.text			= "UPDATE user SET "
								+ "lastSync = CURRENT_TIMESTAMP "
								+ "WHERE userid = :userid ";
	try {
		stLocal.stmt = sqlRun(stLocal.text, {userid:userid});
	}catch(e) {
		debug("Error: sqlSessionGet: " + e);
		stReturn.success		= false;
		stReturn.message 		= 'Database error';
	}

	return stReturn;
}

// =========================================
// ================ SESSION ================
// =========================================
// ------------ sqlSessionGet ------------
function sqlSessionGet(){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'Session exists';

	stLocal.text			= "SELECT userName, password FROM session ";

	try {
		stLocal.stmt = sqlRun(stLocal.text);
	} catch(e) {
		debug("Error: sqlSessionGet: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc		 	= stLocal.results.data.length;
		stReturn.results	= stLocal.results.data[0];
	} else {
		stReturn.success		= false;
		stReturn.message		= 'Invalid User';
	}

	return stReturn;
}

// ------------ sqlSessionSet ------------
function sqlSessionSet( username, password, remember ){
	var stLocal			= {};

	// clear current session
	sqlSessionClear();

	if( remember == true ){
		stLocal.text			= "INSERT INTO session VALUES ( :username, :password )";
		try {
			stLocal.stmt = sqlRun(stLocal.text, {username:username, password:password});
		} catch (e) {
			debug("Error: sqlSessionSet: " + e);
		}
	}
}

// ------------ sqlSessionClear ------------
function sqlSessionClear(){
	var stLocal					= {};

	stLocal.text			= "DELETE FROM session";
	try {
		stLocal.stmt = sqlRun(stLocal.text);
	}catch(e) {
		debug("Error: sqlSessionClear: " + e);
	}
}


// =========================================
// ============= NOTE FUNCTION =============
// =========================================
// ------------ sqlNotesGet ------------
function sqlNotesGet( userId, groupSize, groupNumber, sort_type, filter ){
	var stReturn					= {};
	var stLocal						= {};

	stReturn.success				= true;
	stReturn.results				= '';
	stReturn.message				= 'Notes queried succesfully';
	stReturn.rc						= 0;

	stLocal.stmt					= new air.SQLStatement();
	stLocal.stmt.sqlConnection		= dB;

	//stmt.addEventListener( air.SQLErrorEvent.ERROR, doNoteGetError );

	stLocal.myOffset				= new Date().getTimezoneOffset() / (-60);

	stLocal.text				= "SELECT noteId, content, noteWasFloating, CASE WHEN tag IS NULL OR tag = '' THEN '+tag' ELSE tag END as tag, '" + nameSpace.noteInfo.defImage + "' as image, CAST(datetime( dateStamp, '" + stLocal.myOffset + " hours' ) as varchar) as dateStamp, type, archived FROM note "
	 								+ "WHERE userId = :userId "
								 	+ "AND toDelete = 0 ";

	if( typeof filter != 'undefined' && filter != '' ){
		stLocal.text += "AND " + filter + " ";
	}
	else{
		stLocal.text += "AND archived = 0 ";
	}
	stLocal.text				+= "ORDER BY dateStamp DESC";
	/*if( sort_type == 'chrono' ){
		stLocal.text			+=  "ORDER BY dateStamp DESC";
	}
	else{
		stLocal.text			+=  "ORDER BY type DESC, dateStamp DESC";
	}*/
	if( groupSize != 0 ){
		stLocal.text			+=  " LIMIT " + groupSize;
	}

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
			userId: userId
		});
	} catch(e) {
		debug("Error: sqlNotesGet: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data;
	} else {
		stReturn.success		= false;
		stReturn.message		= 'No notes found for this user';
	}
	return stReturn;
}

// ------------ sqlNoteGetByShifdNoteId ------------
function sqlNoteGetByShifdNoteId( shifdNoteId ){
	var stReturn					= {};
	var stLocal						= {};

	stReturn.success				= true;
	stReturn.results				= '';
	stReturn.message				= 'Notes queried succesfully';
	stReturn.rc						= 0;



	stLocal.myOffset				= new Date().getTimezoneOffset() / (-60);

	stLocal.text				= "SELECT noteId, content, CASE WHEN tag IS NULL OR tag = '' THEN '+tag' ELSE tag END as tag, '" + nameSpace.noteInfo.defImage + "' as image, CAST(datetime( dateStamp, '" + stLocal.myOffset + " hours') as varchar) as dateStamp, type, archived FROM note "
	 									+ "WHERE shifdNoteId = :shifdNoteId ";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
			shifdNoteId: shifdNoteId
		});
	} catch(e) {
		debug("Error: sqlNoteGetByShifdNoteId: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0];
	} else {
		stReturn.success		= false;
		stReturn.message		= 'No notes found for this user';
	}
	return stReturn;
}

function sqlGetPreviousNoteByDateStamp(userId, dateStamp, typeFilter) {
	var stReturn					= {};
	var stLocal						= {};

	stReturn.success				= true;
	stReturn.results				= '';
	stReturn.message				= '';

	stLocal.text				= "SELECT noteId  FROM note WHERE userId = :userId AND datetime(dateStamp) > :dateStamp "
									+ typeFilter
									+ "ORDER BY dateStamp LIMIT 1";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
			userId: userId,
			dateStamp: dateStamp
		});
	} catch(e) {
		debug("Error: sqlGetPreviousNoteByDateStamp: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0];
	} else {
		stReturn.success		= false;
		stReturn.message		= 'Note not found';
	}
	return stReturn;

}

// ------------ sqlGetNote ------------
function sqlGetNote( noteId ){
	var stReturn					= {};
	var stLocal						= {};

	stReturn.success				= true;
	stReturn.results				= '';
	stReturn.message				= 'Notes queried succesfully';

	stLocal.myOffset				= new Date().getTimezoneOffset() / (-60);

	stLocal.text				= "SELECT noteId, userId, shifdNoteId, content, CASE WHEN tag IS NULL OR tag = '' THEN '+tag' ELSE tag END as tag, '" + nameSpace.noteInfo.defImage + "' as image, archived, toDelete, toUpdate, CAST(datetime( dateStamp ) as varchar) as dateStamp, CAST(datetime( dateStamp, '" + stLocal.myOffset + " hours' ) as varchar) as localDateStamp, type, CAST(datetime(date_modified) as varchar) as date_modified FROM note "
	 									+ "WHERE noteId = :noteId ";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
			noteId: noteId
		});
	} catch(e) {
		debug("Error: sqlGetNote: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0];
	} else {
		stReturn.success		= false;
		stReturn.message		= 'Notes not found';
	}
	return stReturn;
}


function sqlGetNoteFloatPosition( noteId ){
	var stReturn					= {};
	var stLocal						= {};

	stReturn.success				= true;
	stReturn.results				= '';
	stReturn.message				= 'Notes queried succesfully';

	stLocal.text				= "SELECT noteId, lastFloatNotePos FROM note "
	 								+ "WHERE noteId = :noteId ";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
			noteId: noteId
		});
	} catch(e) {
		debug("Error: sqlGetNote: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc				= stLocal.results.data.length;
		stReturn.results		= stLocal.results.data[0].lastFloatNotePos;
	} else {
		stReturn.success		= false;
		stReturn.message		= 'Notes not found';
	}
	return stReturn;
}

function sqlSaveNoteFloatPosition( noteId , floatPos ){
	var stReturn					= {};
	var stLocal						= {};

	stReturn.success				= true;
	stReturn.results				= '';
	stReturn.message				= 'Notes updated';

	stLocal.text				= "UPDATE note SET lastFloatNotePos=:lastFloatNotePos "
	 									+ "WHERE noteId = :noteId ";

	try {
		stLocal.stmt = sqlRun(stLocal.text,{
			lastFloatNotePos: floatPos,
			noteId: noteId
		});
	} catch(e) {
		debug("Error: sqlSaveNoteFloatPosition: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	return stReturn;
}

function sqlSaveNoteIsFloating( noteId , floating ){
	var stReturn					= {};
	var stLocal						= {};

	stReturn.success				= true;
	stReturn.results				= '';
	stReturn.message				= 'Notes updated';

	stLocal.text				= "UPDATE note SET noteWasFloating=:floating "
	 									+ "WHERE noteId = :noteId ";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
			floating: floating,
			noteId: noteId
		});
	} catch(e) {
		debug("Error: sqlSaveNoteIsFloating: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	return stReturn;
}

// ------------ sqlNoteGetToSend ------------
function sqlNoteSetToSend( userId ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'Notes queried succesfully';


	stLocal.stLastSync			= sqlUserGetLastSync( nameSpace.loginUser.userId );
	stLocal.parameters			= {};

	stLocal.text			= "SELECT noteId, shifdNoteId, type, content, tag, toDelete, toUpdate, archived, CAST(datetime(dateStamp) as varchar) as dateStamp, CAST(datetime(date_modified) as varchar) as date_modified FROM note "
								 + "WHERE userId = :userId "
								 + "AND (  "
								 + "shifdNoteId IS NULL "
								 + "OR toUpdate = 1 "
								 + "OR toDelete = 1 "
	if( stLocal.stLastSync.success && stLocal.stLastSync.results.lastSync != '' ){
		stLocal.text		+= "OR datetime(date_modified) >= :lastSync ";
		stLocal.parameters['lastSync'] = stLocal.stLastSync.results.lastSync
	}
	stLocal.text			+=  " ) ";

	stLocal.parameters['userId'] = userId;

	try {
		stLocal.stmt = sqlRun(stLocal.text, stLocal.parameters);
	} catch(e) {
		debug("ERROR: sqlNoteSetToSend: " + e);
		stReturn.success		= false;
		stReturn.message		= 'Database error';
		return stReturn;
	}
	//
	stLocal.results			= stLocal.stmt.getResult();
	if (stLocal.results.data != null) {
		stReturn.rc 		= stLocal.results.data.length;
		stReturn.results 	= stLocal.results.data;
	} else {
		stReturn.success		= false;
		stReturn.message		= 'No notes found to sync-send for this user';
	}

	return stReturn;
}

// ------------ sqlNoteSetToDelete ------------
function sqlNoteSetToDelete( noteId ){
	var stLocal					= {};


	stLocal.stNote				= sqlGetNote( noteId );

	// check if note exists
	if( stLocal.stNote.success ){
		// if note exists on API, set to delete
		if( stLocal.stNote.results.shifdNoteId != null ){
			stLocal.text	= "UPDATE note SET "
								 + "toDelete = 1, "
								 + "date_modified = CURRENT_TIMESTAMP "
								 + "WHERE noteId = :noteId";
			try {
				stLocal.stmt = sqlRun(stLocal.text,{
					noteId: noteId
				});
			}catch(e) {
				debug("Error: sqlNotesSetToDelete: " + e);
			}
		}
		// delete local
		else{
			sqlNoteDelete( noteId );
		}
	}
	else{
		doAlert( stLocal.stNote.message )
	}
}

// ------------ sqlNoteSetToArchive ------------
function sqlNoteSetToArchive( noteId ){
	var stLocal					= {};


	stLocal.stNote				= sqlGetNote( noteId );

	// check if note exists
	if( stLocal.stNote.success ){
		// if note exists on API, set to delete
		if( stLocal.stNote.results.shifdNoteId != null ){
			stLocal.text	= "UPDATE note SET "
			 					+ "archived = 1, "
			 					+ "date_modified = CURRENT_TIMESTAMP "
			 					+ "WHERE noteId = :noteId";
			try {
				stLocal.stmt = sqlRun(stLocal.text, {
					noteId: noteId
				});
			}catch(e) {
				debug("Error: sqlNoteSaveByShifdNoteId: " + e);
			}
		}
		// delete local
		else{
			sqlNoteDelete( noteId );
		}
	}
	else{
		doAlert( stLocal.stNote.message )
	}
}

// ------------ sqlNoteSaveByShifdNoteId ------------
function sqlNoteSaveByShifdNoteId( stNote ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'Notes unarchived succesfully';


	// check shifdNoteId
	if( typeof stNote.shifdNoteId != 'undefined' && typeof stNote.userId != 'undefined' ){
		stLocal.text		= "SELECT noteId FROM note "
								 + "WHERE userId = :userId "
								 + "AND shifdNoteId = :shifdNoteId ";

		try{
			// throws exceptions
			stLocal.stmt = sqlRun(stLocal.text, {
				userId: stNote.userId,
				shifdNoteId: stNote.shifdNoteId
			});
		}
		catch(e){
			debug("Error: sqlNoteSaveByShifdNoteId: " + e);
			stReturn.success	= false;
			stReturn.results	= '';
			stReturn.message	= 'Database error.';
			return stReturn;
		}

		stLocal.oNote		= stLocal.stmt.getResult();
		if (stLocal.oNote.data != null) {
			stLocal.rc 			= stLocal.oNote.data.length;
			stReturn.results 	= stLocal.oNote.data[0].noteId;
			stNote.noteId 		= stReturn.results;
		} else {
			stReturn.success	= false;
			stReturn.results	= '';
			stReturn.message	= 'Note unarchived failed: note not found.';
		}

	}
	else{
		stReturn.success		= false;
		stReturn.results		= '';
		stReturn.message		= 'Note unarchived failed: Shifd Note Id not provided.';
	}

	// if note exists, save
	if( stReturn.success == true && typeof stNote.noteId != 'undefined' ){
		return sqlNoteSave( stNote );
	}
	return stReturn;
}

// ------------ sqlNoteSave ------------
function sqlNoteSave( stNote ){
	var stReturn				= {};
	var stLocal					= {};

	stReturn.success			= true;
	stReturn.results			= '';
	stReturn.message			= 'Notes saved succesfully';

	// check shifdNoteId
	if( typeof stNote.shifdNoteId != 'undefined' && typeof stNote.userId != 'undefined' ){
		stLocal.text		= "SELECT noteId FROM note "
		 						+ "WHERE userId = :userId "
		 						+ "AND shifdNoteId = :shifdNoteId ";

		try {
			stLocal.stmt = sqlRun( stLocal.text, {
				userId: stNote.userId,
				shifdNoteId: stNote.shifdNoteId
			});
		} catch(e) {
			debug("Error: sqlNoteSave1: " + e);
			stReturn.success			= false;
			stReturn.message			= 'Database error';
			return stReturn;
		}
		stLocal.oNote			= stLocal.stmt.getResult();
		if (stLocal.oNote.data != null) {
			stLocal.rc 			= stLocal.oNote.data.length;
			stReturn.results 	= stLocal.oNote.data[0].noteId;
			stNote.noteId 		= stReturn.results;
		}
	}

	stLocal.parameters = {};

	// insert
	if( ( typeof stNote.userId != 'undefined' && typeof stNote.type != 'undefined' && typeof stNote.content != 'undefined' ) && ( typeof stNote.noteId == 'undefined' || stNote.noteId == '' || stNote.noteId == 0 ) ){

		//stLocal.stmt.text		= "BEGIN TRANSACTION; "
		stLocal.text		= "INSERT INTO note "
		stLocal.text		=  stLocal.text + "( userId, type, content "
		if( typeof stNote.shifdNoteId != 'undefined' ){
			stLocal.text	= stLocal.text + ",shifdNoteId "
		}
		if( typeof stNote.tag != 'undefined' ){
			stLocal.text	= stLocal.text + ",tag "
		}
		if( typeof stNote.archived != 'undefined' ){
			stLocal.text	= stLocal.text + ",archived "
		}
		if( typeof stNote.dateStamp != 'undefined' ){
			stLocal.text	= stLocal.text + ",dateStamp "
		}
		if( typeof stNote.date_modified != 'undefined' ){
			stLocal.text	= stLocal.text + ",date_modified "
		}
		stLocal.text		= stLocal.text + ") VALUES "
		stLocal.text		= stLocal.text + "( :userId, " +
													":type, " +
													":content ";

		stLocal.parameters["userId"] = stNote.userId;
		stLocal.parameters["type"] = stNote.type;
		stLocal.parameters["content"] = stNote.content;

		if( typeof stNote.shifdNoteId != 'undefined' ){
			stLocal.text	= stLocal.text + ",:shifdNoteId "
			stLocal.parameters["shifdNoteId"] = stNote.shifdNoteId;
		}
		if( typeof stNote.tag != 'undefined' ){
			stLocal.text	= stLocal.text + ",:tag "
			stLocal.parameters["tag"] = stNote.tag;
		}
		if( typeof stNote.archived != 'undefined' ){
			stLocal.text	= stLocal.text + ",:archived  "
			stLocal.parameters["archived"] = stNote.archived == 1;
		}
		if( typeof stNote.dateStamp != 'undefined' ){
			stLocal.text	= stLocal.text + ",:dateStamp "
			stLocal.parameters["dateStamp"] = stNote.dateStamp;
		}
		if( typeof stNote.date_modified != 'undefined' ){
			stLocal.text	= stLocal.text + ",:date_modified "
			stLocal.parameters["date_modified"] = stNote.date_modified;
		}
		stLocal.text		= stLocal.text + ") "

		try {
			stLocal.stmt = sqlRun(stLocal.text, stLocal.parameters);
		} catch(e){
			debug("Error: sqlNoteSave: " + e);
			stReturn.success 		= false;
			stReturn.message		= 'Database error';
			return stReturn;
		}
		stLocal.oSavedNote		= stLocal.stmt.getResult();
		if (stLocal.oSavedNote.lastInsertRowID != null) {
			stLocal.rc 			= 1;
			stReturn.results 	= stLocal.oSavedNote.lastInsertRowID;
		}

	}
	// update
	else if( typeof stNote.noteId != 'undefined' ){
		stLocal.text = "UPDATE	note SET "
		stLocal.text = stLocal.text + "noteId = noteId "
		if( typeof stNote.shifdNoteId != 'undefined' ){
			stLocal.text = stLocal.text + ",shifdNoteId = :shifdNoteId "
			stLocal.parameters["shifdNoteId"] = stNote.shifdNoteId;
		}
		if( typeof stNote.content != 'undefined' ){
			stLocal.text = stLocal.text + ",content = :content "
			stLocal.parameters["content"] = stNote.content;
		}
		if( typeof stNote.tag != 'undefined' ){
			stLocal.text = stLocal.text + ",tag = :tag "
			stLocal.parameters["tag"] = stNote.tag;
		}
		if( typeof stNote.type != 'undefined' ){
			stLocal.text = stLocal.text + ", type = :type "
			stLocal.parameters["type"] = stNote.type;
		}
		if( typeof stNote.archived != 'undefined' ){
			stLocal.text = stLocal.text + ",archived = :archived "
			stLocal.parameters["archived"] = stNote.archived == 1;
		}
		if( typeof stNote.toDelete != 'undefined' ){
			stLocal.text = stLocal.text + ",toDelete = :toDelete "
			stLocal.parameters["toDelete"] = stNote.toDelete == 1;
		}
		if( typeof stNote.toUpdate != 'undefined' ){
			stLocal.text = stLocal.text + ",toUpdate = :toUpdate "
			stLocal.parameters["toUpdate"] = stNote.toUpdate == 1;
		}
		if( typeof stNote.dateStamp != 'undefined' ){
			stLocal.text = stLocal.text + ",dateStamp = :dateStamp "
			stLocal.parameters["dateStamp"] = stNote.dateStamp;
		}
		if( typeof stNote.date_modified != 'undefined' ){
			stLocal.text = stLocal.text + ",date_modified = :date_modified "
			stLocal.parameters["date_modified"] = stNote.date_modified;
		}
		else{
			if( typeof stNote.toUpdate != 'undefined' || typeof stNote.archived != 'undefined' ){
				stLocal.text = stLocal.text + ",date_modified = CURRENT_TIMESTAMP "
			}
		}
		stLocal.text	= stLocal.text + "WHERE noteId = :noteId "
		stLocal.parameters["noteId"] = stNote.noteId;
		try {
			stLocal.stmt = sqlRun(stLocal.text, stLocal.parameters);
		}catch(e) {
			debug("Error: sqlNoteSave: " + e);
			stReturn.success 		= false;
			stReturn.message		= 'Database error';
			return stReturn;
		}
		stReturn.results	= stNote.noteId;
	}
	return stReturn;
}

// ------------ sqlNoteDelete ------------
function sqlNoteDelete( noteId ){
	var stLocal					= {};

	stLocal.text			= "DELETE FROM note WHERE noteId = :noteId"
	try  {
		stLocal.stmt = sqlRun(stLocal.text, {
			noteId: noteId
		});
	}catch(e) {
		debug("SQLError: sqlNoteDelete" + e);
	}
}

function sqlNotesCount(userId) {
	var stReturn					= {};
	var stLocal						= {};

	stReturn.success				= true;

	stLocal.text				= "SELECT count(*) as count FROM note WHERE userId = :userId";

	try {
		stLocal.stmt = sqlRun(stLocal.text, {
			userId: userId
		});
	} catch(e) {
		debug("Error: sqlNotesCount: " + e);
		stReturn.success		= false;
		return stReturn;
	}
	stLocal.oResult		= stLocal.stmt.getResult();
	stReturn.count 		= stLocal.oResult.data[0].count;
	//
	return stReturn;
}

// =========================================
// ============ GLOBAL FUNCTION ============
// =========================================
// hash
function doHash( stringToHash ){
	return stringToHash;
}



// ================================================
// ============ GLOBAL SQL WITH PARAMS ============
// ================================================
var sqlPreparedStatements = {};
function sqlRun(sql, params) {
	var stmt = sqlPreparedStatements[sql];
	debug('running sql statement : ' + sql);
	if (!stmt) {
		stmt = new air.SQLStatement();
		stmt.addEventListener( air.SQLErrorEvent.ERROR, doStmtError );
		stmt.text = sql;
		stmt.sqlConnection = dB;
		sqlPreparedStatements[sql] = stmt;
		debug('created new sql statement : '+sql);
	}
	var pars = "";
	if(params){
		stmt.clearParameters();
		if(params.length && params.item){ //this is an array
			for(var i = 0,l = params.length;i < l;i ++){
				stmt.parameters[i]=params[i];
			}
		}else{
			for(var i in params){
				if(typeof params[i] != 'undefined'){
					stmt.parameters[':' + i] = params[i];
					pars += ':' + i + " = " + params[i] + "(" + typeof params[i] + ")";
				}
			}
		}
	}
	debug("Parameters: " + pars);
	stmt.execute();
	return stmt;
}

function setShowInput(userid, state) {
	var stLocal = {};

	stLocal.sql = "UPDATE user SET "
		+ "showInput = :state "
		+ "WHERE userid = :userid ";

	stLocal.stmt = new air.SQLStatement();
	try {
		stLocal.stmt = sqlRun(stLocal.sql, {
								  state: state,
								  userid: userid
							  });
	}catch(e) {
		debug("Error: setShowInput: " + e);
	}

}

function getShowInput(userid) {
	var stLocal = {};
	var stReturn = {};
	stReturn.success = true;
	stLocal.sql = "SELECT showInput FROM user WHERE userId = :userId";
	stLocal.stmt = new air.SQLStatement();
	try {
		stLocal.stmt = sqlRun(stLocal.sql, {
								  userId: userid
							  });
	}catch(e) {
		debug("Error: getShowInput: " + e);
		stReturn.success = false;
		return stReturn;
	}
	stLocal.oResult = stLocal.stmt.getResult();
	stReturn.showInput = stLocal.oResult.data[0].showInput;
//	trace("SHOW INPUT: " + stReturn.showInput);
	return stReturn;
}
